-- a filtered list of all the items that should be shown
local GNFP_GuildNewsTable = {};

local AddonLoadedHook = CreateFrame("Frame")
AddonLoadedHook:RegisterEvent("ADDON_LOADED");
AddonLoadedHook:SetScript("OnEvent", function(self, event, AddonName)
	if AddonName == "Blizzard_GuildUI" then
		GNFP_DB = GNFP_DB or {};
		GBFP_ReplaceBlizzardFunctions()
		GNFP_SetupOptions()
		GNFP_InitFilterRanks()
	end
end)

function GNFP_SetupOptions()
	--GuildNewsFiltersFrame
	
-- Make the frame larger to fit in the options
	GuildNewsFiltersFrame:SetHeight(GuildNewsFiltersFrame:GetHeight()+120)
	
	local backdrop = {
		bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",  -- path to the background texture
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",  -- path to the border texture
		tile = false,    -- true to repeat the background texture to fill the frame, false to scale it
		tileSize = 32,  -- size (width or height) of the square repeating background tiles (in pixels)
		edgeSize = 10,  -- thickness of edge segments and square size of edge corners (in pixels)
		insets = {    -- distance from the edges of the frame to those of the background texture (in pixels)
			left = 4,
			right = 4,
			top = 4,
			bottom = 4
		}
	}
	
-- Min Item Level Frame (begin) ------------------------------------------------------------------------------------------------------
	GNFP_MinItemLevel_Frame=CreateFrame("FRAME","GNFP_MinItemLevel_Frame",GuildNewsFiltersFrame);
	GNFP_MinItemLevel_Frame:SetWidth(50); 
	GNFP_MinItemLevel_Frame:SetHeight(20);
	GNFP_MinItemLevel_Frame:SetPoint("TopLeft",GuildNewsFiltersFrame,"TopLeft",150, -242);
	GNFP_MinItemLevel_Frame:SetBackdrop(backdrop)
-- Min Item Level Frame (end) ------------------------------------------------------------------------------------------------------

-- Min Item Level Text (begin) ------------------------------------------------------------------------------------------------------
	GNFP_MinItemLevel_Text = GNFP_MinItemLevel_Frame:CreateFontString("WPug_DifferentTypes_Frame_Text","ARTWORK","GameFontNormal");
	GNFP_MinItemLevel_Text:SetPoint("Right",GNFP_MinItemLevel_Frame,"Left",-5,0);
	GNFP_MinItemLevel_Text:SetJustifyH("Right");
	GNFP_MinItemLevel_Text:SetText("|cffffffff" .. GNF_MIN_ITEM_LEVEL .. ":");
-- Min Item Level Text (end) ------------------------------------------------------------------------------------------------------

-- Min Item Level CheckBox (begin) ------------------------------------------------------------------------------------------------------
	GNFP_MinItemLevel_CheckButton = CreateFrame("CheckButton", "GNFP_MinItemLevel_CheckButton", GuildNewsFiltersFrame, "UICheckButtonTemplate");
	GNFP_MinItemLevel_CheckButton:SetWidth(23);
	GNFP_MinItemLevel_CheckButton:SetHeight(23);
	GNFP_MinItemLevel_CheckButton:SetPoint("TopLeft",GuildNewsFiltersFrame,"TopLeft",25, -242);
	GNFP_MinItemLevel_CheckButton:SetText("");
	GNFP_MinItemLevel_CheckButton:SetChecked(GNFP_DB["GNFP_MinItemLevel_CheckButton"])
	GNFP_MinItemLevel_CheckButton:SetScript("OnClick", function(self)
		GNFP_DB[self:GetName()] = self:GetChecked();
		GuildNewsSort(0);
	end)
-- Min Item Level CheckBox (end) ------------------------------------------------------------------------------------------------------
	
-- Min Item Level (begin) ------------------------------------------------------------------------------------------------------
	GNFP_MinItemLevel_Box = CreateFrame("EditBox", "GNFP_MinItemLevel_Box", GuildNewsFiltersFrame, InputBoxTemplate )
	GNFP_MinItemLevel_Box:SetFont("Fonts\\FRIZQT__.TTF", 12)
	GNFP_MinItemLevel_Box:SetAutoFocus(false)
	GNFP_MinItemLevel_Box:SetNumeric(true)
	GNFP_MinItemLevel_Box:SetWidth(460)
	GNFP_MinItemLevel_Box:SetHeight(12) -- Set these to whatever height/width is needed 
	GNFP_MinItemLevel_Box:SetPoint("Left", GNFP_MinItemLevel_Frame, "Left", 8, 0)
	
	GNFP_MinItemLevel_Box:SetText(GNFP_DB["GNFP_MinItemLevel_Box"] or "354")
	GNFP_MinItemLevel_Box:SetScript("OnEditFocusLost",	function(self)
													if GNFP_MinItemLevel_Box:GetText() == "" then
														GNFP_MinItemLevel_Box:SetText(0)
													end
													
													GNFP_DB[self:GetName()] = self:GetText();
													GuildNewsSort(0);
												end)
	GNFP_MinItemLevel_Box:SetScript("OnEnterPressed",	function(self) self:ClearFocus() 	end)
	GNFP_MinItemLevel_Box:SetScript("OnEscapePressed",	function(self) self:ClearFocus()	end)
-- Min Item Level (end) ------------------------------------------------------------------------------------------------------



-- Ignored Words Frame (begin) ------------------------------------------------------------------------------------------------------
	GNFP_IgnoredWords_Frame=CreateFrame("FRAME","GNFP_IgnoredWords_Frame",GuildNewsFiltersFrame);
	GNFP_IgnoredWords_Frame:SetWidth(90); 
	GNFP_IgnoredWords_Frame:SetHeight(20);
	GNFP_IgnoredWords_Frame:SetPoint("TopLeft",GuildNewsFiltersFrame,"TopLeft",150, -266);
	GNFP_IgnoredWords_Frame:SetBackdrop(backdrop)
-- Ignored Words Frame (end) ------------------------------------------------------------------------------------------------------

-- Ignored Words Text (begin) ------------------------------------------------------------------------------------------------------
	GNFP_IgnoredWords_Text = GNFP_IgnoredWords_Frame:CreateFontString("WPug_DifferentTypes_Frame_Text","ARTWORK","GameFontNormal");
	GNFP_IgnoredWords_Text:SetPoint("Right",GNFP_IgnoredWords_Frame,"Left",-5,0);
	GNFP_IgnoredWords_Text:SetJustifyH("Right");
	GNFP_IgnoredWords_Text:SetText("|cffffffff" .. GNF_IGNORED_WORDS .. ":");
	
	GNFP_IgnoredWords_Text2 = GNFP_IgnoredWords_Frame:CreateFontString("WPug_DifferentTypes_Frame_Text","ARTWORK","GameFontNormal");
	GNFP_IgnoredWords_Text2:SetPoint("TopLeft",GNFP_IgnoredWords_Text,"BottomLeft",-25,-5);
	GNFP_IgnoredWords_Text2:SetJustifyH("Right");
	GNFP_IgnoredWords_Text2:SetText("|cff9f9f9f(".. GNF_IGNORED_WORD_INFO ..")");
-- Ignored Words Text (end) ------------------------------------------------------------------------------------------------------

-- Ignored Words CheckBox (begin) ------------------------------------------------------------------------------------------------------
	GNFP_IgnoredWords_CheckButton = CreateFrame("CheckButton", "GNFP_IgnoredWords_CheckButton", GuildNewsFiltersFrame, "UICheckButtonTemplate");
	GNFP_IgnoredWords_CheckButton:SetWidth(23);
	GNFP_IgnoredWords_CheckButton:SetHeight(23);
	GNFP_IgnoredWords_CheckButton:SetPoint("TopLeft",GuildNewsFiltersFrame,"TopLeft",25, -264);
	GNFP_IgnoredWords_CheckButton:SetText("");
	GNFP_IgnoredWords_CheckButton:SetChecked(GNFP_DB["GNFP_IgnoredWords_CheckButton"])
	GNFP_IgnoredWords_CheckButton:SetScript("OnClick", function(self)
		GNFP_DB[self:GetName()] = self:GetChecked();
		GuildNewsSort(0);
	end)
-- Ignored Words CheckBox (end) ------------------------------------------------------------------------------------------------------
	
-- Ignored Words (begin) ------------------------------------------------------------------------------------------------------
	GNFP_IgnoredWords_Box = CreateFrame("EditBox", "GNFP_IgnoredWords_Box", GuildNewsFiltersFrame, InputBoxTemplate )
	GNFP_IgnoredWords_Box:SetFont("Fonts\\FRIZQT__.TTF", 12)
	GNFP_IgnoredWords_Box:SetAutoFocus(false)
	GNFP_IgnoredWords_Box:SetWidth(80)
	GNFP_IgnoredWords_Box:SetHeight(12) -- Set these to whatever height/width is needed 
	GNFP_IgnoredWords_Box:SetPoint("Left", GNFP_IgnoredWords_Frame, "Left", 8, 0)
	
	GNFP_IgnoredWords_Box:SetText(GNFP_DB["GNFP_IgnoredWords_Box"] or "")
	GNFP_IgnoredWords_Box:SetScript("OnEditFocusLost",	function(self)
													GNFP_DB[self:GetName()] = self:GetText();
													GuildNewsSort(0);
												end)
	GNFP_IgnoredWords_Box:SetScript("OnEnterPressed",	function(self) self:ClearFocus() 	end)
	GNFP_IgnoredWords_Box:SetScript("OnEscapePressed",	function(self) self:ClearFocus()	end)
-- Ignored Words (end) ------------------------------------------------------------------------------------------------------

-- Filter Ranks DropDown Button (begin) ------------------------------------------------------------------------------------------------------
	GNFP_FilterRanks_Button = CreateFrame("Button", "GNFP_FilterRanks_Button", GuildNewsFiltersFrame, "UIPanelButtonTemplate");
	GNFP_FilterRanks_Button:SetWidth(80);
	GNFP_FilterRanks_Button:SetPoint("TopLeft",GuildNewsFiltersFrame,"TopLeft",150, -216);
	GNFP_FilterRanks_Button:SetText(GNF_EDIT_FILTER_RANKS);
	GNFP_FilterRanks_Button:SetScript("OnClick", function(self)
		ToggleDropDownMenu(1, nil,GNFP_FilterRanks_DropDown , self:GetName(), 0, 0)
	end)	
-- Filter Ranks DropDown Button (end) ------------------------------------------------------------------------------------------------------

-- FIlter Ranks Text (begin) ------------------------------------------------------------------------------------------------------
	GNFP_FilterRanks_Text = GNFP_FilterRanks_Button:CreateFontString("WPug_DifferentTypes_Frame_Text","ARTWORK","GameFontNormal");
	GNFP_FilterRanks_Text:SetPoint("Right",GNFP_FilterRanks_Button,"Left",-5,0);
	GNFP_FilterRanks_Text:SetJustifyH("Right");
	GNFP_FilterRanks_Text:SetText("|cffffffff" .. GNF_FILTER_RANKS .. ":");
	
-- Filter Ranks Text (end) ------------------------------------------------------------------------------------------------------

-- Filter Ranks CheckBox (begin) ------------------------------------------------------------------------------------------------------
	GNFP_FilterRanks_CheckBox = CreateFrame("CheckButton", "GNFP_FilterRanks_CheckBox", GuildNewsFiltersFrame, "UICheckButtonTemplate");
	GNFP_FilterRanks_CheckBox:SetWidth(23);
	GNFP_FilterRanks_CheckBox:SetHeight(23);
	GNFP_FilterRanks_CheckBox:SetPoint("TopLeft",GuildNewsFiltersFrame,"TopLeft",25, -216);
	GNFP_FilterRanks_CheckBox:SetText("");
	GNFP_FilterRanks_CheckBox:SetChecked(GNFP_DB["GNFP_FilterRanks_CheckBox"])
	GNFP_FilterRanks_CheckBox:SetScript("OnClick", function(self)
		GNFP_DB[self:GetName()] = self:GetChecked();
		GNFP_CollectGuildMemberInfo();
		GuildNewsSort(0);
	end)
-- Filter Ranks CheckBox (end) ------------------------------------------------------------------------------------------------------
	if(GNFP_DB["GNFP_FilterRanks_CheckBox"]) then
		GNFP_CollectGuildMemberInfo();
	end

end

function GNFP_CollectGuildMemberInfo()
	local totalMembers, onlineMembers, onlineAndMobileMembers = GetNumGuildMembers();
	for roosterIndex = 1, totalMembers do -- go through guild roster to create database (only way to associate guild members name with their rank (if there is another way this might be doable alot more efficiently))
		local fullName, rank, rankIndex, level, class, zone, note, officernote, online, isAway, classFileName, achievementPoints, achievementRank, isMobile = GetGuildRosterInfo(roosterIndex);
		local charName, _ = strsplit("-", fullName, 2) -- delete Server portion, necessary as server names will sometimes not be loaded, might cause confusion if exact names appear multiple times in one guild
		if not(GNFP_DB["memberRank" .. charName] == rankIndex) then --should be more efficient as most write operations will be skipped
			GNFP_DB["memberRank" .. charName] = rankIndex;
		end
	end
end

function GNFP_IgnoredWordsFilter(itemName)
	-- reset the filter out flag
	local GNFP_FilterFlag = true;

	if GNFP_IgnoredWords_Box:GetText() ~= "" then
		local SearchText = GNFP_IgnoredWords_Box:GetText();
		
		while SearchText ~= nil do
			if SearchText ~= nil then
				local SearchTerm, RestOfText = strsplit(",", SearchText, 2)
				SearchText = RestOfText;
				
				if itemName ~= nil and SearchTerm ~= nil then
					if strfind(strlower(itemName), strlower(SearchTerm)) then
						--set the filter out flag for this id to false (it is not allowed)
						GNFP_FilterFlag = false;
					end
				end
			end
		end
	end
	
	return GNFP_FilterFlag
end

function GNFP_InitFilterRanks() -- filtered Rank selection DropDown Menu (begin)

	GNFP_FilterRanks_DropDown = CreateFrame("FRAME", "GNFPFilterRanksDropDown", UIParent, "UIDropDownMenuTemplate"); -- creating the dropdown frame
	UIDropDownMenu_SetWidth(GNFP_FilterRanks_DropDown, 50);
	UIDropDownMenu_SetText(GNFP_FilterRanks_DropDown, "This should not be visible!"); -- as this menu is only opened via ToggleDropDownMenu this should not be visible
	UIDropDownMenu_Initialize(GNFP_FilterRanks_DropDown, function(self, level, menuList) -- setting the dropdown menus initialization function to:
		local info = UIDropDownMenu_CreateInfo(); -- create info
		-- add Title (begin)
		info.isTitle,info.notCheckable = true, true; -- set info attributes to fit title format
	    info.text = GNF_IGNORED_RANKS .. ":"; -- set title text
	    UIDropDownMenu_AddButton(info); -- add title to dropdown menu
		info.isTitle, info.notCheckable, info.disabled = false, false, false; -- reset info attributes to keep using it for normal buttons
		-- add Title (end)
		for rankLevel = 1, GuildControlGetNumRanks() do --go through all ranks
			info.text, info.checked = GuildControlGetRankName(rankLevel), GNFP_DB["filteredRank" .. rankLevel]; -- read rank filter settings from DB
			info.func = self.addRankFilter; -- add function to button (see below)
			info.arg1 = rankLevel; -- set first argument of function to rankLevel
			info.arg2 = info.checked; -- set second argument of function to whether this rank was filtered already
			UIDropDownMenu_AddButton(info); -- add button to dropdown menu
		end
	end)

	function GNFP_FilterRanks_DropDown:addRankFilter(rankLevel, filtered) -- function is called when a FilterRanks_DropdDown option is clicked
		if(filtered) then -- if the rankLevel is currently filtered out:
			GNFP_DB["filteredRank" .. rankLevel] = false; -- set the option whether to filter the rankLevel out to false
		else -- if the rankLevel is currently not filtered out:
			 GNFP_DB["filteredRank" .. rankLevel] = true; -- set the option whether to filter the rankLevel out to true
		end 
		GuildNewsSort(0); -- update the news display
		ToggleDropDownMenu(1, nil,GNFP_FilterRanks_DropDown, GNFP_FilterRanks_Button, 0, 0); --reopen the dropdown menu (make it seem like it never closed ;) )
	end

end -- filtered Rank selection DropDown Menu (end)

function GBFP_ReplaceBlizzardFunctions()
	local orig_GetGuildNewsInfo = GetGuildNewsInfo;
	local orig_GetNumGuildNews  = GetNumGuildNews;
	local orig_GuildNewsButton_OnClick = GuildNewsButton_OnClick;
	
	function GetNumGuildNews()
		local j = 0;
		GNFP_GuildNewsTable = {};
		
		local NewsIsAllowed = true;
		
		for i=1,orig_GetNumGuildNews() do 
			local isSticky, isHeader, newsType, text1, text2, id, data, data2, weekday, day, month, year = orig_GetGuildNewsInfo(i);
		
			if newsType == 1 or newsType == 3 or newsType == 4 or newsType == 5 then
			
				NewsIsAllowed = true;
			
				if NewsIsAllowed and GNFP_FilterRanks_CheckBox:GetChecked() and text1 then
					local memberName, _ = strsplit("-", text1, 2) -- delete Server portion, necessary as server names will sometimes not be loaded, might cause confusion if two exact names exist in one guild
					if GNFP_DB[("filteredRank" .. ((GNFP_DB["memberRank" .. memberName] or (-1)) + 1))] then --check if members rank is filtered out or not (or -1 is used if guild member has left the guild)
						NewsIsAllowed = false;
					end
				end
				if newsType == 3 or newsType == 4 or newsType == 5 then
					
					local ItemStringAllowed = GNFP_IgnoredWordsFilter(text2);
				
					-- word filter
					if GNFP_IgnoredWords_CheckButton:GetChecked() and NewsIsAllowed  then
						if not ItemStringAllowed then
							NewsIsAllowed = false;
						end
					end
					
					--item level filter
					if GNFP_MinItemLevel_CheckButton:GetChecked() and NewsIsAllowed and text2 ~= nil then
						--Get the item level from the item id
						local name, link, quality, iLevel, reqLevel, class, subclass, maxStack, equipSlot, texture, vendorPrice = GetItemInfo(text2);
						iLevel = iLevel or 0;
						
						-- get the compare value from the box
						local itemLevelCompare = tonumber(GNFP_MinItemLevel_Box:GetText()) or 0;
					
						--check if it should be filtered
						if iLevel < itemLevelCompare then
							NewsIsAllowed = false;
						end
					end
				end
				--If the item is allowed then add it to the directory
				if NewsIsAllowed then
					j = j + 1;
					GNFP_GuildNewsTable[j] = i;
				end
			else
				j = j + 1;
				GNFP_GuildNewsTable[j] = i;
			end
		end
		
		return j;
	end
	
	function GetGuildNewsInfo(index)		
		local isSticky, isHeader, newsType, text1, text2, id, data, data2, weekday, day, month, year = orig_GetGuildNewsInfo(GNFP_GuildNewsTable[index]);

		if newsType == 3 or newsType == 4 or newsType == 5 and text2 ~= nil then

			local name, link, quality, iLevel, reqLevel, class, subclass, maxStack, equipSlot, texture, vendorPrice = GetItemInfo(text2);
			iLevel = iLevel or 0;

			text1 = "(|cFFF2C319" .. iLevel .. "|r) " .. text1

		end
		
		return isSticky, isHeader, newsType, text1, text2, id, data, data2, weekday, day, month, year
		--return orig_GetGuildNewsInfo(GNFP_GuildNewsTable[index])
	end
	
	
	
	function GuildNewsButton_OnClick(self, button)
		if button == "LeftButton" then
			if IsControlKeyDown() or IsShiftKeyDown() then				
				local isSticky, isHeader, newsType, text1, text2, id, data, data2, weekday, day, month, year = GetGuildNewsInfo(self.index);
				
				if newsType == 3 or newsType == 4 or newsType == 5 and text2 ~= nil then
					local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount,itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(text2);
					
					if IsShiftKeyDown() then
						ChatEdit_InsertLink(itemLink) 
					elseif IsControlKeyDown() then
						DressUpItemLink(itemLink)
					end
				end
			end
		end
		
		orig_GuildNewsButton_OnClick(self, button)
    end
end

